//
// Copyright (c) 2016, MindFusion LLC - Bulgaria.
//

using System;
using System.Drawing;
using System.Windows.Forms;


namespace MindFusion.Scheduling.WinForms.Samples.CS.AdvancedViewers
{
	/// <summary>
	/// A tree view class capable of creating snapshots of itself.
	/// </summary>
	public class TreeViewEx : TreeView
	{
		/// <summary>
		/// Generates an image of the current state of this view.
		/// The image needs to be disposed by the caller.
		/// </summary>
		public Image GenerateImage()
		{
			Graphics temp = CreateGraphics();
			Bitmap b = new Bitmap(this.ClientRectangle.Width,
				this.ClientRectangle.Height, temp);
			temp.Dispose();

			Graphics graphics = Graphics.FromImage(b);
			IntPtr dc = graphics.GetHdc();

			Message m = new Message();
			m.Msg = Win32.WM_PAINT;
			m.HWnd = Handle;
			m.WParam = dc;
			m.LParam = IntPtr.Zero;

			base.DefWndProc(ref m);

			graphics.ReleaseHdc(dc);
			graphics.Dispose();

			return b;
		}

		/// <summary>
		/// Returns the node from this tree corresponding
		/// to the specified handle.
		/// </summary>
		public TreeNode NodeFromHandle(IntPtr nodeHandle)
		{
			return NodeFromHandle(Nodes, nodeHandle);
		}

		/// <summary>
		/// Recursively searches the nodes from the specified collection
		/// and their children for the node corresponding to the
		/// specified handle.
		/// </summary>
		private static TreeNode NodeFromHandle(TreeNodeCollection nodes, IntPtr nodeHandle)
		{
			foreach (TreeNode node in nodes)
			{
				if (node.Handle == nodeHandle)
					return node;

				TreeNode current = NodeFromHandle(node.Nodes, nodeHandle);
				if (current != null)
					return current;
			}

			return null;
		}
	}
}
